/***************************************************************************/
/*                                                                         */
/*                        M Y J O I N E N G I N E                          */
/*                                                                         */
/***************************************************************************/




#include "MyJoinEngine.h"
#include "../../MyFrameAl5/Obj/Boundary.h"
#include "../../MyFrameAl5/Math/CalcHelper.h"
#include "../../MyFrameAl5/FrameGlobalVars.h"
#include "GlobalDefs.h"

/*-------------------------------------------------------------------------*/
/*-------------------------- Init new join player -------------------------*/
/*-------------------------------------------------------------------------*/
CJoinPlayer* init_new_join_player()
{
	return (CJoinPlayer*)(new CMyJoinPlayer());
}

/*-------------------------------------------------------------------------*/
/*---------------------------- Init join engine ---------------------------*/
/*-------------------------------------------------------------------------*/
void init_join_engine()
{	
	ASSERT(_pJoinEngine == NULL);
	_pJoinEngine = new CJoinEngine();
	_pJoinEngine->SetMenusEnabled(
		0x00
	//	JOINENGMENU_WAITOTHERP				
		|JOINENGMENU_SETGAMEMODE
	//	|JOINENGMENU_SETDIFFICULTY	
	//	|JOINENGMENU_SETCHARACTER	
	//	|JOINENGMENU_SETPROPERTIES
	);
	
	// disable SET_GAMEMODE_MENU If NbPlayers == 1
	_pJoinEngine->AddStepRestriction(CJoinMenuStepRetriction(JOINENGMENU_SETGAMEMODE, false, -1, 1));

	// enable SET_DIFFICULTY_MENU Only If GameMode == 0
	_pJoinEngine->AddStepRestriction(CJoinMenuStepRetriction(JOINENGMENU_SETDIFFICULTY, true, 0, -1));	

	_pJoinEngine->SetGameMetrics(CGameMetrics(0, 0, 0), init_new_join_player, NULL);
	_pJoinEngine->Reset();
}

/*-------------------------------------------------------------------------*/
/*---------------------------- CMy Join Player ----------------------------*/
/*-------------------------------------------------------------------------*/
CMyJoinPlayer::CMyJoinPlayer():CJoinPlayer()
{
	m_fYTime  = 0.8f*BITMAP_H;
	m_fYTitle = 0.1f*BITMAP_H;
	m_fYVal   = 0.5f*BITMAP_H;

}

/*-------------------------------------------------------------------------*/
/*---------------------------- ~ CMy Join Player --------------------------*/
/*-------------------------------------------------------------------------*/
/* virtual */ CMyJoinPlayer::~CMyJoinPlayer()
{
}

/*-------------------------------------------------------------------------*/
/*-------------------------------- Init Menu ------------------------------*/
/*-------------------------------------------------------------------------*/
/* virtual */ void CMyJoinPlayer::InitMenu(int nMenuType)
{
	switch (GetCurrentMenuType())
	{
	case JOINENGMENU_WAITOTHERP:		
		break;

	case JOINENGMENU_SETGAMEMODE	:
		InitMenuCommon(3, 1, "_|_|_", NULL);	
		break;

	case JOINENGMENU_SETDIFFICULTY	:
		InitMenuCommon(2, 1, "_|_", NULL);
		break;

	case JOINENGMENU_SETCHARACTER	:
		InitMenuCommon(m_pParent->m_GameMetrics.m_nCharacterMax, 1, NULL, NULL);
		break;

	case JOINENGMENU_SETPROPERTIES	:
		InitMenuCommon(m_pParent->m_GameMetrics.m_nPropertiesMaxItems, 
			m_pParent->m_GameMetrics.m_nPropertiesMaxLevelups, "_|_|_|_", NULL);
		break;

	case JOINENGMENU_SETSTARTSTAGE	:
		InitMenuCommon(10, 1, NULL, NULL);
		break;
	}
}

/*-------------------------------------------------------------------------*/
/*----------------------------- Get Text Alpha ----------------------------*/
/*-------------------------------------------------------------------------*/
float CMyJoinPlayer::GetMenuTextAlpha()
{
	float fIntroTime = m_pParent->GetIntroTime();
	ASSERT(fIntroTime >= -1.0f && fIntroTime <= 1.0f);
	return MAX(0.0f, 1.0f-2.0f*fabs(fIntroTime));
}

/*-------------------------------------------------------------------------*/
/*---------------------------- Get Title Alpha ----------------------------*/
/*-------------------------------------------------------------------------*/
float CMyJoinPlayer::GetTitleAlpha()
{
	if (m_pParent->GetIntroTime() > 0.0f)
		return 1.0f;

	return GetMenuTextAlpha();
}

/*-------------------------------------------------------------------------*/
/*------------------------------ Get Text Lit -----------------------------*/
/*-------------------------------------------------------------------------*/
float CMyJoinPlayer::GetMenuTextLit()
{
	int t = GetMenuItemElapsedTime();	
	return 1.0f-CalcLinear_Int(t, 0, 16);
}

/*-------------------------------------------------------------------------*/
/*-------------------------------- Set Pos X ------------------------------*/
/*-------------------------------------------------------------------------*/
void CMyJoinPlayer::SetPosX(bool bSeparate)
{
	if (!bSeparate)
	{
		m_fX = 0.5f*BITMAP_W;
	}
	else
	{
		m_fX = 0.25f*BITMAP_W + 0.5f*BITMAP_W*m_nPlayerNum;
	}
}

/*-------------------------------------------------------------------------*/
/*------------------------------- Draw Title ------------------------------*/
/*-------------------------------------------------------------------------*/
void CMyJoinPlayer::DrawTitle(const char* szBuf)
{
	if (strcmp(szBuf, "") != 0)
	{		
		//_pFontNormal->PushMagnify(2.0f, 4.0f);		
		//_pFontNormal->TextoutClip_Alpha(m_fX, m_fYTitle, m_nElapsedTime, m_fTitleAlpha, szBuf, ALLEGRO_ALIGN_CENTER);
		//_pFontNormal->PopMagnify();
	}
}

/*-------------------------------------------------------------------------*/
/*------------------------------ Draw Timeout -----------------------------*/
/*-------------------------------------------------------------------------*/
void CMyJoinPlayer::DrawTimeout()
{
	//char szBuf[32];

	//int t = (m_pParent->m_nTimeout)%FPS_EXPECTED;
	//float fLit = CalcLinear_Int(t, FPS_EXPECTED/2, FPS_EXPECTED);
	//ASSERT(fLit >= 0.0f && fLit <= 1.0f);

	//_pFontNormal->PushMagnify(2.0f, 4.0f);
	//sprintf(szBuf, "TIME %2d", (m_pParent->m_nTimeout)/FPS_EXPECTED);
	//_pFontNormal->TextoutClip_Ex(m_fX, m_fYTime, 999, 0.0f, m_fTitleAlpha, 
	//	fLit, 1.0f, 1.0f, 0.0f, szBuf, ALLEGRO_ALIGN_CENTER);
	//_pFontNormal->PopMagnify();
}

/*-------------------------------------------------------------------------*/
/*-------------------------- Draw Selected Value --------------------------*/
/*-------------------------------------------------------------------------*/
void CMyJoinPlayer::DrawSelectedValue(bool bNumber, bool bText)
{
	//char szBuf[80];
	////_pFontNormal->PushMagnify(2.0f, 4.0f);
	//if (bNumber)
	//{
	//	sprintf(szBuf, "(%d) ", GetCurrentMenuItemVal());
	//}
	//else
	//{
	//	strcpy(szBuf, "");
	//}

	//if (bText)
	//{
	//	strncat(szBuf, GetCurrentMenuItemLabel(), 50);
	//}
	//float fLitR = 1.0f;
	//float fLitG = 0.0f;
	//float fLitB = 0.0f;
	//_pFontNormal->TextoutClip_Ex(m_fX, m_fYVal, m_nElapsedTime, 0.0f, 
	//	m_fTextAlpha, m_fTextLit, fLitR,fLitG,fLitB, szBuf, ALLEGRO_ALIGN_CENTER);
	//_pFontNormal->PopMagnify();
}

/*-------------------------------------------------------------------------*/
/*---------------------------- Draw All Values ----------------------------*/
/*-------------------------------------------------------------------------*/
void CMyJoinPlayer::DrawAllValues()
{
	//char szBuf[80];
	//sprintf(szBuf, "PRESS %c", _pFontNormal->GetCharForButtonNum(JOYBTNNUM_BUTTON1) );
	//_pFontNormal->TextoutClip(m_fX, 0, 999, szBuf, ALLEGRO_ALIGN_CENTER);
}

/*-------------------------------------------------------------------------*/
/*---------------------------- Draw Attributes ----------------------------*/
/*-------------------------------------------------------------------------*/
void CMyJoinPlayer::DrawAttributes()
{
}


/*-------------------------------------------------------------------------*/
/*--------------------------- Draw Menu Separate --------------------------*/
/*-------------------------------------------------------------------------*/
/* virtual */ void CMyJoinPlayer::DrawMenu_Separate()
{
	CJoinPlayer::DrawMenu_Separate();
		
	SetPosX(true);
	//m_fTitleAlpha = GetTitleAlpha();
	//m_fTextAlpha = GetMenuTextAlpha();
	//m_fTextLit = GetMenuTextLit();
	//m_nElapsedTime = GetMenuTotalElapsedTime()-10;

	//switch (GetCurrentMenuType())
	//{
	//case JOINENGMENU_WAITOTHERP:
	//	if (m_nJoinStatus == JOINSTA_NOBODY)
	//	{
	//		char szBuf[64];
	//		sprintf(szBuf, "PRESS %dP START\nTO JOIN IN !", m_nPlayerNum+1);			
	//		DrawTitle(szBuf);
	//	}
	//	break;

	//case JOINENGMENU_SETGAMEMODE	:
	//	break;

	//case JOINENGMENU_SETDIFFICULTY	:
	//	break;

	//case JOINENGMENU_SETCHARACTER	:
	//	if (m_nJoinStatus == JOINSTA_HASJOINED)
	//	{
	//		DrawTitle("SELECT YOUR\nCHARACTER");
	//		DrawAllValues();
	//	}
	//	break;

	//case JOINENGMENU_SETPROPERTIES	:
	//	if (m_nJoinStatus == JOINSTA_HASJOINED)
	//	{
	//		DrawTitle("ADAPT YOUR\nATTRIBUTES");
	//		DrawAttributes();		
	//	}
	//	break;

	//case JOINENGMENU_SETSTARTSTAGE	:
	//	break;

	//case JOINENGMENU__ENDSTATE:
	//	return;
	//	break;
	//}
}

/*-------------------------------------------------------------------------*/
/*---------------------------- Draw Menu Share ----------------------------*/
/*-------------------------------------------------------------------------*/
void CMyJoinPlayer::DrawMenu_Share()
{
	SetPosX(false);
	
	// char szBuf[64] = {0};

	//switch (GetCurrentMenuType())
	//{
	//case JOINENGMENU_WAITOTHERP:		
	//	break;

	//case JOINENGMENU_SETGAMEMODE	:
	//	DrawTitle("SELECT GAME MODE");
	//	DrawSelectedValue(false, true);
	//	break;

	//case JOINENGMENU_SETDIFFICULTY	:
	//	DrawTitle("SELECT DIFFICULTY");
	//	DrawSelectedValue(false, true);
	//	break;

	//case JOINENGMENU_SETCHARACTER	:
	//	break;

	//case JOINENGMENU_SETPROPERTIES	:
	//	break;

	//case JOINENGMENU_SETSTARTSTAGE	:
	//	DrawTitle("SELECT STARTING\nSTAGE");
	//	DrawSelectedValue(true, true);
	//	break;

	//case JOINENGMENU__ENDSTATE:
	//	return;
	//	break;
	//}

	// DrawTimeout();
}
