/***************************************************************************/
/*                                                                         */
/*                        P L A Y E R E N T I T Y                          */
/*                                                                         */
/***************************************************************************/





#include "PlayerEntity.h"
#include "MyGameHelper.h"
#include "../../MyFrameAl5/FrameGlobalVars.h"
#include "../../MyFrameAl5/Draw/MyText.h"
#include "../../MyFrameAl5/Draw/DrawPrim.h"
#include "../../MyFrameAl5/Math/CalcHelper.h"
#include "../../MyFrameAl5/Math/MyRand.h"
#include "../../MyFrameAl5/Game/Player.h"
#include "../../MyFrameAl5/Prj/Prj.h"
#include "MyObj.h"

CPlayerEntity::CPlayerEntity():CMyLivingObject()
{
	InitPlayer();
}

CPlayerEntity::~CPlayerEntity()
{
}

void CPlayerEntity::InitPlayer()
{
	CLivingObject::InitHelper(1.0f, 1.0f, OBJTYPE_PLAYERPOD, OWNER_PLAYER1);
	CLivingObject::InitLivingObjectFlags(
		// LIVOBJFLG_FRAMEINVINCIBILITY_KILLED
		// |
		LIVOBJFLG_INVINCIBILITYATSTARTUP
		, PLAYER_NB_LIVES_BEGIN
		, PLAYER_EXPLODERESPAWN_TIME, PLAYER_KILLED_INVINCIBILITY_TIME, -1);

	InitXYWH(_fCameraOffsetCurrentX+0.5f*BITMAP_W	, PLAYER_INIT_Y, PLAYER_COLL_W, PLAYER_COLL_H);
	m_fOriginalX = m_fX;

	InitMovieNumByMovieExternalId(
		0 // MOVIEID_
		, PLAYER_SPR_W, PLAYER_SPR_H);

	SetExternalId(EXTERNALID_PLAYER1);

	m_nMoveTransitionTime = 0;
	m_nDirectionLook = JOYBTNNUM_LEFT;
	m_nIdleMoveTime = 0;
	m_nWalkAnimTime = 0;
}

/*-------------------------------------------------------------------------*/
/*------------------------------ Is Disposable ----------------------------*/
/*-------------------------------------------------------------------------*/
/*virtual*/ bool CPlayerEntity::IsDisposable()
{
	return false;
}

/*-------------------------------------------------------------------------*/
/*---------------------------------- Move ---------------------------------*/
/*-------------------------------------------------------------------------*/
/*virtual*/ int CPlayerEntity::Move()
{
	float CAMERA_DIFF = 160.0f;
	float PLAYER_DX = 2.0f;
	char nCurrentDirection = m_nDirectionLook;

	if (al_key_down(&_keyboard_state, ALLEGRO_KEY_LEFT))
	{
		m_nDirectionLook = JOYBTNNUM_LEFT;
		m_nWalkAnimTime++;
		m_nIdleMoveTime = 0;
		m_nMoveTransitionTime++;
		m_fOriginalX -= PLAYER_DX;
		_fCameraOffsetTargetX = m_fOriginalX - CAMERA_DIFF; 
	}
	else if (al_key_down(&_keyboard_state, ALLEGRO_KEY_RIGHT))
	{
		m_nDirectionLook = JOYBTNNUM_RIGHT;
		m_nWalkAnimTime++;
		m_nIdleMoveTime = 0;
		m_nMoveTransitionTime++;
		m_fOriginalX += PLAYER_DX;
		_fCameraOffsetTargetX = m_fOriginalX + CAMERA_DIFF;
	}
	else
	{
		m_nDirectionLook |= PLAYER_MYDIRECTION__MASK;
		m_nIdleMoveTime++;
		m_nMoveTransitionTime--;
		_fCameraOffsetTargetX = m_fOriginalX;
	}

	if (nCurrentDirection != m_nDirectionLook)
	{
		_fCameraOffsetOldX = _fCameraOffsetCurrentX;
		_nCameraAnimMaxTime = 4.0f + fabs(_fCameraOffsetOldX - _fCameraOffsetTargetX)*1.0f*CAMERA_MAXTIME_PER_TILE16/16.0f;
		_nCameraAnimMaxTime = CLAMP(4, _nCameraAnimMaxTime, 100);
		_nCameraAnimCurrentTime = _nCameraAnimMaxTime;
	}
	else
	{
		if (_nCameraAnimCurrentTime > 0)
			_nCameraAnimCurrentTime--;
	}

	m_nMoveTransitionTime = CLAMP(0, m_nMoveTransitionTime, PLAYER_MOVETRANSITION_MAXTIME);

	float fPos = 
		  0.6f*CalcSquareAccelDeccel_Int(_nCameraAnimCurrentTime, 0, _nCameraAnimMaxTime)
		+ 0.4f*CalcCosineAccel_Int(_nCameraAnimCurrentTime, 0, _nCameraAnimMaxTime);

	_fCameraOffsetCurrentX = _fCameraOffsetTargetX + fPos*(_fCameraOffsetOldX - _fCameraOffsetTargetX);

	OriginalMove();
	UpdateScreenPos();

	if (_nNbMissilesLeft > 0 && IsPressingFire())
	{
		// Launch Missile
		CMissilePlayer* pMissile = new CMissilePlayer();
		pMissile->Init();
		_pListMyPlayerMissiles->Add(pMissile);
	}
	
	return 0;
}

/*-------------------------------------------------------------------------*/
/*---------------------------- Is Pressing Fire ---------------------------*/
/*-------------------------------------------------------------------------*/
/*static*/ bool CPlayerEntity::IsPressingFire()
{
	return (
		_pMouseCtrl->IsPressedButton(0)
		// || al_key_down(&_keyboard_state, ALLEGRO_KEY_LCTRL)
		// || al_key_down(&_keyboard_state, ALLEGRO_KEY_RCTRL)
		// || al_key_down(&_keyboard_state, ALLEGRO_KEY_SPACE)
		);
}

/*-------------------------------------------------------------------------*/
/*------------------------------ Draw On Top ------------------------------*/
/*-------------------------------------------------------------------------*/
/*virtual*/ int CPlayerEntity::DrawOnTop()
{
	
	return 0;
}

/*-------------------------------------------------------------------------*/
/*--------------------------- Get Source Missile --------------------------*/
/*-------------------------------------------------------------------------*/
void CPlayerEntity::GetSourceMissile(float& fNormalizedSourceX, float& fNormalizedSourceY)
{
	fNormalizedSourceX = m_fX;
	fNormalizedSourceY = m_fY;	
}

/*-------------------------------------------------------------------------*/
/*---------------------------------- Draw ---------------------------------*/
/*-------------------------------------------------------------------------*/
/*virtual*/ int CPlayerEntity::Draw()
{
	// calculate anim WALK //////////////////////////////
	char nItemNum;
	float fPosWalkX[PLAYER_SKELETON_TOTALITEMS];
	float fPosWalkY[PLAYER_SKELETON_TOTALITEMS];
	short WALK_PERIOD = 32;
	for (nItemNum=0; nItemNum < PLAYER_SKELETON_ONE_LEG_NBITEMS; nItemNum++)
	{
		float fRadius = (nItemNum+1)*PLAYER_SKELETON_H/PLAYER_SKELETON_ONE_LEG_NBITEMS;
		float fAngle = -45.0f + 90.0f*CalcCosineLoop_Int(m_nWalkAnimTime%WALK_PERIOD, 0, WALK_PERIOD); 
		fAngle += 20.0f*CalcSquareAccel_Int(nItemNum, 0, PLAYER_SKELETON_ONE_LEG_NBITEMS);
		
		fPosWalkX[2*nItemNum  ] = fRadius*get_dx_a360_hq(185.0f + fAngle);
		fPosWalkY[2*nItemNum  ] = fRadius*get_dy_a360_hq(185.0f + fAngle);
		
		fPosWalkX[2*nItemNum+1] = fRadius*get_dx_a360_hq(185.0f - fAngle);
		fPosWalkY[2*nItemNum+1] = fRadius*get_dy_a360_hq(185.0f - fAngle);
	}

	// calculate anim IDLE /////////////////////////////
	float fPosIdleX[PLAYER_SKELETON_TOTALITEMS];
	float fPosIdleY[PLAYER_SKELETON_TOTALITEMS];
	float fIdleAnimPos = CalcCosineAccelDeccel_Int(m_nIdleMoveTime-PLAYER_SKELETON_IDLE_TIME, 0, 64);
	for (nItemNum=0; nItemNum < PLAYER_SKELETON_ONE_LEG_NBITEMS; nItemNum++)
	{
		float fDiffX = 0.4f*PLAYER_SKELETON_W*CalcCosineLoop_Int((m_nIdleMoveTime + nItemNum*7)%40, 0, 40);
		float fTempX = 0.1f*PLAYER_SKELETON_W*nItemNum/PLAYER_SKELETON_ONE_LEG_NBITEMS;
		fPosIdleX[2*nItemNum] = 0.1f*PLAYER_SKELETON_W + fTempX + fIdleAnimPos*(fDiffX - fTempX);
		fPosIdleX[2*nItemNum + 1] = -fPosIdleX[2*nItemNum];
		
		fPosIdleY[2*nItemNum] = fPosIdleY[2*nItemNum + 1] = 
				(nItemNum+1)*PLAYER_SKELETON_H/PLAYER_SKELETON_ONE_LEG_NBITEMS
			  - 4*CalcAbsTriangle_Int((m_nIdleMoveTime + nItemNum*10)%64, 0, 50);
	}

	// mix the 2 pos //////////////////////////////////
	float fPosFinalX[PLAYER_SKELETON_TOTALITEMS];
	float fPosFinalY[PLAYER_SKELETON_TOTALITEMS];
	float fMoveFactor = CalcLinear_Int(m_nMoveTransitionTime, 0, PLAYER_MOVETRANSITION_MAXTIME);
	for (nItemNum=0; nItemNum < PLAYER_SKELETON_TOTALITEMS; nItemNum++)
	{
		fPosFinalX[nItemNum] = fPosIdleX[nItemNum] + fMoveFactor*(fPosWalkX[nItemNum] - fPosIdleX[nItemNum]);
		fPosFinalY[nItemNum] = fPosIdleY[nItemNum] + fMoveFactor*(fPosWalkY[nItemNum] - fPosIdleY[nItemNum]);
	}

	// DRAW HEAD ////////////////////////////////////////////

	float fRectangleHeadW = 48; // CHECK OUT MY RECTANGLE HEAD OVER HERE!!
	float fRectangleHeadH = 24;

	if (_bDead)
	{
		fRectangleHeadW += 32;
		fRectangleHeadH -= 8;
	}

	float fColVal = 0.75f;
	float fHeadAlpha = 0.9f;

	DrawMonoQuad4f(
		  (int)m_fX, (int)(m_fY+0.5f*fRectangleHeadH)
		, fRectangleHeadW+8, fRectangleHeadH+8
		, 0,0,0, 0.5f);

	DrawMonoQuad4f(
		  (int)m_fX, (int)(m_fY+0.5f*fRectangleHeadH)
		, fRectangleHeadW, fRectangleHeadH
		, fColVal,fColVal,fColVal, fHeadAlpha);

	float fMouseX, fMouseY;
	_pMouseCtrl->GetMousePosXY(fMouseX, fMouseY);
	float fEyeAngle = get_nearest_angle(fMouseX-m_fX, fMouseY-m_fY);
	float fEyeWidthAngle = 100.0f;
	float fEyePosMouse = 0.0f;
	bool bEyeSet = false;

	float EYE_BOUND_MIN = 360.0f - 0.5f*fEyeWidthAngle;
	float EYE_BOUND_MAX = 360.0;

	if (   fEyeAngle > EYE_BOUND_MIN
		&& fEyeAngle < EYE_BOUND_MAX)
	{
		fEyePosMouse = CalcPropLinear_FloatFloat(fEyeAngle, EYE_BOUND_MIN, EYE_BOUND_MAX, -1.0f, 0.0f);
		bEyeSet = true;
	}

	EYE_BOUND_MIN = 0.0f;
	EYE_BOUND_MAX = 0.5f*fEyeWidthAngle;

	if (!bEyeSet
		&& fEyeAngle > EYE_BOUND_MIN
		&& fEyeAngle < EYE_BOUND_MAX)
	{
		fEyePosMouse = CalcPropLinear_FloatFloat(fEyeAngle, EYE_BOUND_MIN, EYE_BOUND_MAX, 0.0f, 1.0f);
		bEyeSet = true;
	}

	EYE_BOUND_MIN = 180.0f - 0.5f*fEyeWidthAngle;
	EYE_BOUND_MAX = 180.0f + 0.5f*fEyeWidthAngle;

	if (!bEyeSet
		&& fEyeAngle > EYE_BOUND_MIN
		&& fEyeAngle < EYE_BOUND_MAX)
	{
		fEyePosMouse = CalcPropLinear_FloatFloat(fEyeAngle, EYE_BOUND_MIN, EYE_BOUND_MAX, -1.0f, 1.0f);
		bEyeSet = true;
	}

	EYE_BOUND_MIN = 180.0f + 0.5f*fEyeWidthAngle;
	EYE_BOUND_MAX = 360.0f - 0.5f*fEyeWidthAngle;

	if (!bEyeSet
		&& fEyeAngle > EYE_BOUND_MIN
		&& fEyeAngle < EYE_BOUND_MAX)
	{
		fEyePosMouse = -1.0f;
		bEyeSet = true;
	}

	EYE_BOUND_MIN =        + 0.5f*fEyeWidthAngle;
	EYE_BOUND_MAX = 180.0f - 0.5f*fEyeWidthAngle;

	if (!bEyeSet
		&& fEyeAngle > EYE_BOUND_MIN
		&& fEyeAngle < EYE_BOUND_MAX)
	{
		fEyePosMouse = 1.0f;
		bEyeSet = true;
	}
	
	fEyePosMouse = CLAMP(-1.0f, fEyePosMouse, 1.0f);

	for (char nEyeNum=0; nEyeNum < 2; nEyeNum++)
	{
		// Eye Brow
		float fEyeBrowDx = -0.125f*fRectangleHeadW + 0.5f*fRectangleHeadW*nEyeNum;
		fColVal = 0.4f;
		DrawMonoQuad4f(
			  (int)(m_fX + fEyeBrowDx)
		    , (int)(m_fY +0.425f*fRectangleHeadH)
			, 0.25f*fRectangleHeadW
			, 0.25f*fRectangleHeadH
			, fColVal,fColVal,fColVal, fHeadAlpha);

		// White Eye
		fColVal = 1.0f;
		short ny = (int)(m_fY + 1.0f +0.625f*fRectangleHeadH);
		DrawMonoQuad4f(
			  (int)(m_fX + fEyeBrowDx)
		    , ny
			, 0.25f*fRectangleHeadW
			, 0.25f*fRectangleHeadH
			, fColVal,fColVal,fColVal, fHeadAlpha);

		// Eye Iris
		fColVal = 0.1f;
		float fPupilDx = 0.0625f;
		DrawMonoQuad4f(
			  (int)(m_fX + fEyeBrowDx + CalcPropLinear_FloatFloat(fEyePosMouse, -1.0f, 1.0f, -fPupilDx, fPupilDx)*fRectangleHeadW)
		    , ny
			, 0.125f*fRectangleHeadW
			, 0.250f*fRectangleHeadH
			, fColVal,fColVal,fColVal, fHeadAlpha);
	}

	// DRAW Score /////////////////////////////////////

	int nTmpScore = _nScorePlayer;
	int SCORE_BASE = 8;
	float fBlockW = 8;
	float fGapW = 4;

	float fScoreY = 460;
	CMovieClip* pClip = (CMovieClip*)_pListMovieClips->GetItemByExternalId(MOVIEID_EXPLRECT0);
	unsigned char nScoreSpriteNum = 5 + (_nTotalExeTime/2)%8;

	float fScoreSpriteScaleX = 4.0f;
	float fScoreSpriteScaleY = 2.0f;
	float fScoreSpriteAlpha = 1.0f - 0.75f*(_nTotalExeTime%BEAT_PERIOD)/BEAT_PERIOD;


	while (nTmpScore > 0)
	{
		float fTotalGapW = fBlockW + fGapW;
		float fScoreX = BITMAP_W - 16 - 0.5f*fBlockW;
		fScoreY -= fTotalGapW;
		unsigned char nDigit = nTmpScore%SCORE_BASE;
		nTmpScore /= SCORE_BASE;

		unsigned char i;
		for (i=0; i < nDigit; i++)
		{
			pClip->Draw_RegionWidthScale_NumImage(
				  fScoreX, fScoreY
			    , 0.0f, 0.0f
				, MIN(EXPLRECT_IMAGE_W, fBlockW/fScoreSpriteScaleX)
				, MIN(EXPLRECT_IMAGE_H, fBlockW/fScoreSpriteScaleY)
				, fScoreSpriteScaleX, fScoreSpriteScaleY
				, nScoreSpriteNum
				, fScoreSpriteAlpha
				);

			//DrawMonoQuad4f(
			//	fScoreX, fScoreY, fBlockW, fBlockW,  0,0,1,1);

			fScoreX -= fTotalGapW;
		}

		fBlockW += 4;
	}


	// DRAW sketeton //////////////////////////////////
	float fRectW = 4.0f;
	for (nItemNum=0; nItemNum < PLAYER_SKELETON_TOTALITEMS; nItemNum++)
	{
		for (char nDx=-1; nDx <= 1; nDx+=2)
		{
			short fx_ = (int)(fPosFinalX[nItemNum] + m_fX + 12*nDx);
			short fy_ = (int)(fPosFinalY[nItemNum] + m_fY + nDx + fRectangleHeadH);
			
			DrawMonoQuad4f(
				  fx_ + 2
				, fy_ + 2
				, fRectW, fRectW
				,  0.0f, 0.0f, 0.0f,  0.5f);
				
			DrawMonoQuad4f(
				  fx_
				, fy_
				, fRectW, fRectW
				,  1.0f, 1.0f, 1.0f,  1.0f);
		}
	}
	
	/*--- Debug Info ------------------------------------------------------*/

#ifdef _DEBUG

	if (DEBUG_DISPLAY_FLAGS & DISPFLG_DBG_PLAYERINFO)
	{
		for (float fTmpOriginalX = ORIGINAL_POSX_MIN; fTmpOriginalX < ORIGINAL_POSX_MAX; fTmpOriginalX += 128.0f)
		{
			float fTmpNormalizedX = CMyLivingObject::ConvertOriginalToNormalizedPos_Ex(fTmpOriginalX);
			DrawMonoQuad4f(fTmpNormalizedX, 0.5f*BITMAP_H, 2.0f, 128.0f,  0.0f, 1.0f, 1.0f, 0.5f);

			mytextprintf4ub(fTmpNormalizedX+4, 0.4*BITMAP_H, 0,255,255, 255, "%d"
				, (int)fTmpOriginalX
			);
		}

		mytextprintf4ub(0, 472, 255,255,255, 255, "%5d,%3d,%4d,%4d,%3d,%.1f , %5.1f,%5.2f "
			, (int)m_fX
			, (int)m_fY
			, (int)_fCameraOffsetTargetX
			, (int)_fCameraOffsetCurrentX
			, _nCameraAnimCurrentTime
			, _fGroundLimitDisplayedLives
			, fEyeAngle, fEyePosMouse
		);

		DrawMonoQuad4f(m_fX, m_fY, 16, 16,  0.0f,1.0f,0.0f, 0.8f); // CLivingObject::Draw_CollisionMask();
	}
#endif

	return 0;
}

/*-------------------------------------------------------------------------*/
/*-------------------------------- Draw HUD -------------------------------*/
/*-------------------------------------------------------------------------*/
void CPlayerEntity::DrawHUD()
{
	/*--- Draw ammunition left --------------------------------------------*/
	float fRectH = 16.0f;
	float fRectW = 8.0f;
	float fGapW = 12.0f;

	if (_nNbMissilesLeft > 0)
	{
		float fx = 0.5f*BITMAP_W - 0.5f*fGapW*_nNbMissilesLeft;
		float fy = 0.2f*BITMAP_H;
		float fColR, fColG, fColB;
		unsigned short nBlinkPeriod = 0;

		if (_nNbMissilesLeft <= 3)
		{
			nBlinkPeriod = BEAT_PERIOD >> (3-_nNbMissilesLeft);
		}

		if (nBlinkPeriod > 0 
			&& (int)(_nTotalExeTime%nBlinkPeriod) < (int)nBlinkPeriod/2)
		{
			fColR = 1.0f;
			fColG = 0.0f;
			fColB = 0.0f;
		}
		else
		{
			fColR = 1.0f;
			fColG = 1.0f;
			fColB = 1.0f;
		}
		
		char nItems = _nNbMissilesLeft;
		while (nItems > 0)
		{
			nItems--;
			fx += fGapW;

			DrawMonoQuad4f(fx, fy, fRectW, fRectH,  fColR, fColG, fColB, 0.8f);
		}

	}

}