/***************************************************************************/
/*                                                                         */
/*                                M Y O B J                                */
/*                                                                         */
/***************************************************************************/




#include "myobj.h"
#include "myGameHelper.h"
#include "../../MyFrameAl5/Math/CalcHelper.h"
#include "../../MyFrameAl5/Math/MyRand.h"
#include "MyLivingObject.h"
#include "../../MyFrameAl5/FrameGlobalVars.h"
#include "PlayerEntity.h"



unsigned char _nForcedRandItems[ FORCED_RAND_ITEMS ] = { 
	  3, 76, 121, 12, 181, 49, 230, 20, 134, 241, 98, 203, 34, 191, 81, 71, 149
    , 120, 147, 65, 255, 52, 250, 35, 246, 69, 57, 48, 162, 125, 60, 225, 173, 44
	, 77, 56, 168, 198, 150, 224, 46, 113, 72, 18, 37, 247, 214, 213, 33, 175, 106
	, 8, 155, 186, 128, 86, 118, 232, 39, 40, 174, 108, 104, 68, 59, 4, 119, 244, 2
	, 152, 237, 126, 176, 110, 94, 115, 109, 61, 219, 31, 30, 240, 138, 182, 153, 217
	, 234, 122, 9, 91, 17, 63, 80, 243, 179, 211, 27, 212, 6, 190, 101, 158, 252, 195
	, 188, 238, 99, 114, 24, 160, 111, 178, 180, 249, 223, 87, 25, 141, 93, 156, 146
	, 88, 169, 78, 185, 89, 5, 167, 123, 23, 154, 201, 16, 221, 19, 51, 196, 107, 144
	, 228, 133, 13, 245, 161, 116, 208, 229, 70, 62, 54, 235, 127, 43, 132, 177, 38
	, 136, 251, 82, 145, 226, 209, 205, 42, 220, 103, 157, 170, 159, 75, 200, 29, 135
	, 130, 231, 15, 248, 36, 67, 53, 129, 143, 148, 166, 193, 194, 55, 95, 22, 210, 58
	, 163, 10, 202, 96, 84, 206, 242, 73, 11, 236, 1, 14, 227, 97, 90, 124, 28, 189, 172
	, 140, 165, 83, 218, 215, 0, 64, 50, 112, 92, 66, 171, 105, 164, 183, 74, 85, 45, 184
	, 21, 253, 192, 26, 79, 7, 100, 117, 199, 142, 207, 216, 41, 151, 102, 139, 187, 222
	, 32, 239, 47, 131, 137, 233, 197, 254, 204
};



/*-------------------------------------------------------------------------*/
/*-------------------------------- CAsteroid ------------------------------*/
/*-------------------------------------------------------------------------*/
CAsteroid::CAsteroid():CMyLivingObject()
{	
}

/*-------------------------------------------------------------------------*/
/*------------------------------ ~ CAsteroid ------------------------------*/
/*-------------------------------------------------------------------------*/
CAsteroid::~CAsteroid()
{
}

/*-------------------------------------------------------------------------*/
/*------------------------------ Is Disposable ----------------------------*/
/*-------------------------------------------------------------------------*/
bool CAsteroid::IsDisposable()
{
	return (m_fLife <= 0.0f);
}

/*-------------------------------------------------------------------------*/
/*---------------------------------- Init ---------------------------------*/
/*-------------------------------------------------------------------------*/
void CAsteroid::Init()
{
	int nHardDensity = 
		- 32
		+ 96*CalcSquareDeccel_Int(_nNbAsteroidsSpawned, 20, 100)
		+ 16*CalcLinear_Int(_nNbAsteroidsSpawned, 100, 200)
		;

	int nItemNum;
	
	if (_pGameHelper->IsDemo())
	{
		nItemNum = rand()%FORCED_RAND_ITEMS;
	}
	else
	{
		nItemNum = _nForcedRandItems[_nNbAsteroidsSpawned%FORCED_RAND_ITEMS];		
	}
	
	float fSpeed = 0.8f;
	if (nItemNum < nHardDensity)
	{
		if (nItemNum+10 >= nHardDensity/4)
		{
			// big
			fSpeed *= 1.4f;			
		}
		else
		{
			// CRITICAL
			fSpeed *= 2.1f;
		}
	}

	m_fOriginalX = rand()%( (int)ORIGINAL_POSX_MAX );
	float fAngle = 135.0f + 90.0f*(nItemNum%16)/15;

	InitXYWH(-999, -0.05f*BITMAP_H, ASTEROID_W, ASTEROID_W);
	UpdateScreenPos();

	float fLife = 1.0f;
	float fStrength = 1.0f;
	InitHelper(fLife, fStrength, OBJTYPE_ASTEROID, OWNER_CPU);
	InitLivingObjectFlags(0x00);
	
	InitSpeed(fSpeed, fAngle);
	_nNbAsteroidsSpawned++;


	float fPos = 
		  0.3f*CalcCosineAccelDeccel_Int(_nNbAsteroidsSpawned, 0, 108)
		+ 0.4f*CalcSquareAccel_Int(_nNbAsteroidsSpawned, 10, 50)
		+ 0.3f*CalcSquareAccelDeccel_Int(_nNbAsteroidsSpawned, 40, 120);

	_nNextAsteroidCurrentTime = _nNextAsteroidMaxTime = 
		ASTEROID_IDLE_MAXTIME - (ASTEROID_IDLE_MAXTIME-ASTEROID_IDLE_MINTIME)*fPos;
}

/*-------------------------------------------------------------------------*/
/*---------------------------------- Move ---------------------------------*/
/*-------------------------------------------------------------------------*/
/*virtual*/ int CAsteroid::Move()
{
	OriginalMove();
	UpdateScreenPos();
	return 0;
}

/*-------------------------------------------------------------------------*/
/*---------------------------------- Draw ---------------------------------*/
/*-------------------------------------------------------------------------*/
/*virtual*/ int CAsteroid::Draw()
{
	Draw_Asteroid_Ex(m_fX, m_fY, 1.0f);
	return 0; 
}

/*-------------------------------------------------------------------------*/
/*---------------------------- Draw Asteroid Ex ---------------------------*/
/*-------------------------------------------------------------------------*/
void CAsteroid::Draw_Asteroid_Ex(float fx, float fy, float fForceAlpha)
{
	float fOuterW = 16 + 16*CalcLinear_Int(_nTotalExeTime%BEAT_PERIOD, 0, BEAT_PERIOD);
	float fAlpha = 0.5f*(1.0f - CalcLinear_Int(_nTotalExeTime%BEAT_PERIOD, 0, 0.8f*BEAT_PERIOD));

	DrawMonoQuad4f(fx, fy, fOuterW, fOuterW, 1.0f, 1.0f, 1.0f,  fAlpha*fForceAlpha);
	DrawMonoQuad4f(fx, fy, ASTEROID_W, ASTEROID_W, 1.0f, 1.0f, 1.0f,  1.0f*fForceAlpha);
}

////////////////////////////////////////////////////////////////////////////
//
// MISSILE PLAYER
//
////////////////////////////////////////////////////////////////////////////

/*-------------------------------------------------------------------------*/
/*-------------------------------- CMissilePlayer -------------------------*/
/*-------------------------------------------------------------------------*/
CMissilePlayer::CMissilePlayer():CMyLivingObject()
{	
}

/*-------------------------------------------------------------------------*/
/*------------------------------ ~ CMissilePlayer -------------------------*/
/*-------------------------------------------------------------------------*/
CMissilePlayer::~CMissilePlayer()
{
}

/*-------------------------------------------------------------------------*/
/*------------------------------ Is Disposable ----------------------------*/
/*-------------------------------------------------------------------------*/
bool CMissilePlayer::IsDisposable()
{
	return (m_fLife <= 0.0f);
}

/*-------------------------------------------------------------------------*/
/*---------------------------------- Init ---------------------------------*/
/*-------------------------------------------------------------------------*/
void CMissilePlayer::Init()
{
	float fNormalizedSourceX = 0.5f*BITMAP_W;
	float fNormalizedSourceY = 0.75*BITMAP_H;
	float fNormalizedTargetX, fNormalizedTargetY;
	_pMouseCtrl->GetMousePosXY(fNormalizedTargetX, fNormalizedTargetY);
	float fDist;
	float fDiffX = 100;
	float fDiffY = -100;
	float fAngle360 = 0.0f;

	if (_pListPlayers->GetLength() > 0)
	{
		CPlayerEntity* pMan = (CPlayerEntity*)_pListPlayers->GetItem(0);
		pMan->GetSourceMissile(fNormalizedSourceX, fNormalizedSourceY);
		fDiffX = fNormalizedTargetX - fNormalizedSourceX;
		fDiffY = fNormalizedTargetY - fNormalizedSourceY;
		fAngle360 = get_nearest_angle_and_dist(fDiffX, fDiffY, fDist);
	}
	else
	{
		fDist = 0.5f*BITMAP_H;
	}

	float SPEED_PER_TILE64 = 10;
	m_nTargetMaxTime = fDist*SPEED_PER_TILE64/64.0f;
	
	m_nTargetMaxTime /= BEAT_PERIOD; // scale to BEAT
	m_nTargetMaxTime *= BEAT_PERIOD;
	
	m_nTargetMaxTime += BEAT_PERIOD; // round up

	m_nTargetMaxTime += (BEAT_PERIOD - _nTotalExeTime%BEAT_PERIOD); // delay to next BEAT
	ASSERT((_nTotalExeTime + m_nTargetMaxTime)%BEAT_PERIOD == 0);

	m_nTargetCurrentTime = m_nTargetMaxTime;
	
	m_fOriginalX = ConvertNormalizedToOriginalPos_Ex(fNormalizedSourceX);
	m_nOriginalSourceX = m_fOriginalX;
	m_nOriginalSourceY = (int)fNormalizedSourceY;
	m_nOriginalDiffX = (int)fDiffX;
	m_nOriginalDiffY = (int)fDiffY;

	InitXYWH(fNormalizedSourceX, fNormalizedSourceY, MISSILEPLAYER_COLL_W, MISSILEPLAYER_COLL_W);
	UpdateScreenPos();

	float fLife = 1.0f;
	float fStrength = 1.0f;
	InitHelper(fLife, fStrength, OBJTYPE_PLAYERMISSILE, OWNER_PLAYER1);
	InitLivingObjectFlags(0x00);
	
	m_fRot360 = fAngle360;
	m_fSpeed = 0.0f;
	m_fDx = 0.0f;
	m_fDy = 0.0f;
	_nNbMissilesLeft--;
	m_nMissileNumber = _pListMyPlayerMissiles->GetAddCount();
}

/*-------------------------------------------------------------------------*/
/*---------------------------------- Move ---------------------------------*/
/*-------------------------------------------------------------------------*/
/*virtual*/ int CMissilePlayer::Move()
{
	bool bSpawnExplosion = false;
	if (m_nTargetCurrentTime > 0)
	{
		m_nTargetCurrentTime--;

		if (m_nTargetCurrentTime <= BEAT_PERIOD
			&& m_nTargetCurrentTime%(BEAT_PERIOD/4) == 0)
		{
			unsigned char nTchiCode = (m_nMissileNumber%16) ^ 0x0A;
			if (nTchiCode == 0)
				nTchiCode = 0x0E;


			switch (m_nTargetCurrentTime/(BEAT_PERIOD/4))
			{
			case 1:
				if (nTchiCode & 0x01)
				{
					_pSoundEngine->PlayAnySoundInGroup_Ex(SNDGRPNUM_FXP_TCHI, -32, 0.75f);
				}
				break;

			case 2:
				if (nTchiCode & 0x02)
				{
					_pSoundEngine->PlayAnySoundInGroup_Ex(SNDGRPNUM_FXP_TCHI, 32, 1.0f);
				}
				break;

			case 4:
				if (nTchiCode & 0x08)
				{
					_pSoundEngine->PlayAnySoundInGroup_Ex(SNDGRPNUM_FXP_TCHI, 32, 1.0f);
				}
				break;

			case 3:
				if (nTchiCode & 0x04)
				{
					_pSoundEngine->PlayAnySoundInGroup_Ex(SNDGRPNUM_FXP_TCHI, -32, 0.8f);
				}
				break;
			}
		}

		float fPos = 1.0f - CalcSquareAccel_Int(m_nTargetCurrentTime, 0, m_nTargetMaxTime);
		m_fOriginalX = m_nOriginalSourceX + fPos*m_nOriginalDiffX;
		m_fY = m_nOriginalSourceY + fPos*m_nOriginalDiffY;

		OriginalMove();
	}
	else
	{
		m_fLife = 0.0f;
		bSpawnExplosion = true;
	}
	
	UpdateScreenPos();

	if (bSpawnExplosion)
	{
		CMyExplosion* pExpl = new CMyExplosion();
		pExpl->Init(m_fOriginalX, m_fY, 0, 0, true);
		_pListMyPlayerExplosions->Add(pExpl);

		ASSERT(IsDisposable());
	}

	return 0;
}

/*-------------------------------------------------------------------------*/
/*---------------------------------- Draw ---------------------------------*/
/*-------------------------------------------------------------------------*/
/*virtual*/ int CMissilePlayer::Draw()
{
	float fOuterW = MISSILEPLAYER_COLL_W*(0.8f + 0.7f*CalcLinear_Int(_nTotalExeTime%BEAT_PERIOD, 0, BEAT_PERIOD));
	float fAlpha = 0.5f*(1.0f - CalcLinear_Int(_nTotalExeTime%BEAT_PERIOD, 0, 0.8f*BEAT_PERIOD));

	DrawMonoQuad4f(m_fX, m_fY, fOuterW, fOuterW, 1.0f, 1.0f, 1.0f,  fAlpha);
	DrawMonoQuad4f(m_fX, m_fY, MISSILEPLAYER_COLL_W, MISSILEPLAYER_COLL_W, 1.0f, 1.0f, 1.0f,  0.3f);
	DrawMonoQuad4f(m_fX, m_fY, MISSILEPLAYER_SPR_W, MISSILEPLAYER_SPR_W, 1.0f, 1.0f, 1.0f,  1.0f);
	return 0; 
}

////////////////////////////////////////////////////////////////////////////
//
// MY EXPLOSION
//
////////////////////////////////////////////////////////////////////////////

CMyExplosion::CMyExplosion():CMyLivingObject()
{
}

/*-------------------------------------------------------------------------*/
/*---------------------------- ~ CMy Explosion ----------------------------*/
/*-------------------------------------------------------------------------*/
/*virtual*/ CMyExplosion::~CMyExplosion()
{
}

/*-------------------------------------------------------------------------*/
/*------------------------------ Is Disposable ----------------------------*/
/*-------------------------------------------------------------------------*/
/*virtual*/ bool CMyExplosion::IsDisposable()
{
	return (m_nAnimCurrentTime >= m_nAnimMaxTime);
}

/*-------------------------------------------------------------------------*/
/*---------------------------------- Init ---------------------------------*/
/*-------------------------------------------------------------------------*/
void CMyExplosion::Init(int nOriginalX, int nOriginalY, int nExplosionLevel, int nDelay, bool bSound)
{
	m_nExplosionLevel = nExplosionLevel;
	
	if (nDelay >= 0)
	{
		m_nSpawnDelay = nDelay;
	}
	else
	{
		// auto
		m_nSpawnDelay = (BEAT_PERIOD - _nTotalExeTime%BEAT_PERIOD); // delay to next BEAT
	}

	if (bSound)
	{
		ASSERT((_nTotalExeTime + m_nSpawnDelay)%BEAT_PERIOD == 0);
	}

	m_nAnimCurrentTime = 0;
	m_nAnimMaxTime = 4*BEAT_PERIOD;

	m_fOriginalX = nOriginalX; 
	float fNormalizedSourceX = ConvertOriginalToNormalizedPos_Ex(m_fOriginalX);

	InitXYWH(fNormalizedSourceX, nOriginalY, EXPLRECT_COLL_W, EXPLRECT_COLL_H);
	UpdateScreenPos();

	if (m_nSpawnDelay > 0)
	{
		m_fW = ASTEROID_W;
		m_fH = ASTEROID_W;
	}

	float fLife = 1.0f;
	float fStrength = 1.0f;
	InitHelper(fLife, fStrength, OBJTYPE_MYEXPLOSION, OWNER_PLAYER1);
	InitLivingObjectFlags(0x00);
	
	m_fRot360 = 0.0f;
	m_fSpeed = 0.0f;
	m_fDx = 0.0f;
	m_fDy = 0.0f;
	InitMovieNumByMovieExternalId(MOVIEID_EXPLRECT0 + rand()%EXPLRECT_MAXMOVIES, 16, 16);
}

/*-------------------------------------------------------------------------*/
/*---------------------------------- Move ---------------------------------*/
/*-------------------------------------------------------------------------*/
/*virtual*/ int CMyExplosion::Move()
{
	if (m_nSpawnDelay > 0)
	{
		m_nSpawnDelay--;
	}
	else
	{
		if (m_nAnimCurrentTime == 0)
		{
			_pSoundEngine->PlayAnySoundInGroup_Object(SNDGRPNUM_FXP_BEAT, this);

			int nSoundDelay = BEAT_PERIOD/4;
			int nDiff = BEAT_PERIOD/2;
			int nTmpLevel = m_nExplosionLevel;
			while (nTmpLevel > 0)
			{
				_pSoundEngine->PlayAnySoundInGroup_Object(SNDGRPNUM_FXP_CLAP, this, 1.0f, nSoundDelay);
				nSoundDelay += nDiff;
				nDiff = BEAT_PERIOD/4;
				nTmpLevel--;
			}
		}

		m_nAnimCurrentTime++;
		m_fW = EXPLRECT_COLL_W;
		m_fH = EXPLRECT_COLL_H;
	}

	UpdateScreenPos();
	return 0;
}

/*-------------------------------------------------------------------------*/
/*-------------------------------- Collide --------------------------------*/
/*-------------------------------------------------------------------------*/
/*virtual*/ int CMyExplosion::Collide(CLivingObject *pOtherAsteroid)
{
	int nRet = LIVOBJ_SUFFER_DIED;
	if (!IsDisposable())
	{
		nRet = CLivingObject::Collide(pOtherAsteroid);
		ASSERT(pOtherAsteroid->IsDisposable());
		ASSERT(!IsDisposable());

		// Spawn another Explosion
		CMyExplosion* pExpl = new CMyExplosion();
		pExpl->Init(((CAsteroid*)pOtherAsteroid)->m_fOriginalX, pOtherAsteroid->m_fY, m_nExplosionLevel+1, -1, true);
		_pListMyPlayerExplosions->Add(pExpl);

		
		// Reward IF COMBOed/Explosion-Level : More ammo
		if (m_nExplosionLevel > 0)
		{
			_nNbMissilesLeft++;
		}

		// Reward IF LastCollide was chained : EVEN MORE AMMO!!!1!
		if (_nLastCollideExplosion < 4*BEAT_PERIOD)
		{
			_nNbMissilesLeft++;
			_nScorePlayer++;
		}

		_nScorePlayer += (m_nExplosionLevel+1);

		_nLastCollideExplosion = 0;
	}

	return nRet;
}

/*-------------------------------------------------------------------------*/
/*---------------------------------- Draw ---------------------------------*/
/*-------------------------------------------------------------------------*/
/*virtual*/ int CMyExplosion::Draw()
{
	if (m_nSpawnDelay > 0)
	{
		CAsteroid::Draw_Asteroid_Ex(m_fX, m_fY, 0.5f);
	}
	else
	{
		short RECT_SPAWN_TIME = 10;

#ifdef _DEBUG
		short nSpriteNumMax = RECT_SPAWN_TIME + EXPLRECT_MOVIE_MAXSPRITESANIM;
		ASSERT(m_nAnimMaxTime >= nSpriteNumMax);
#endif
		float EXPL_ALPHA_MAX = 0.8f;

		if (m_nAnimCurrentTime < RECT_SPAWN_TIME)
		{
			float fScaleX = 0.10f + 0.90f*CalcSquareDeccel_Int(m_nAnimCurrentTime, 0, RECT_SPAWN_TIME);
			float fScaleY = 0.15f + 0.85f*CalcLinear_Int(m_nAnimCurrentTime, 0, RECT_SPAWN_TIME);
			float fAlpha  = 0.50f + 0.50f*CalcLinear_Int(m_nAnimCurrentTime, 0, RECT_SPAWN_TIME);
			DrawMonoQuad4f(
				  m_fX, m_fY
				, fScaleX*m_fW, fScaleY*m_fH
				, 1.0f, 1.0f, 1.0f
				, EXPL_ALPHA_MAX*fAlpha
				);
		}
		else
		{
			int nSpriteNum = EXPLRECT_MOVIE_MAXSPRITESANIM*CalcLinear_Int(m_nAnimCurrentTime, RECT_SPAWN_TIME, m_nAnimMaxTime);
			_pListMovieClips->Draw_W_H_ClipNum(
				  GetMovieClipNum()
				, m_fX, m_fY
				, m_fW, m_fH
				, 0.0f
				, nSpriteNum, 0.5f
				, EXPL_ALPHA_MAX
				);
		}
	}
	

	return 0;
}


////////////////////////////////////////////////////////////////////////////
//
// CITY
//
////////////////////////////////////////////////////////////////////////////

CCityRect::CCityRect():CMyLivingObject()
{
}

/*-------------------------------------------------------------------------*/
/*------------------------------ ~ CCity Rect -----------------------------*/
/*-------------------------------------------------------------------------*/
/*virtual*/ CCityRect::~CCityRect()
{
}

/*-------------------------------------------------------------------------*/
/*------------------------------ Is Disposable ----------------------------*/
/*-------------------------------------------------------------------------*/
/*virtual*/ bool CCityRect::IsDisposable()
{
	return (m_nCollapseAnimTime == 0);
}

/*-------------------------------------------------------------------------*/
/*---------------------------------- Init ---------------------------------*/
/*-------------------------------------------------------------------------*/
void CCityRect::Init(short nOriginX)
{
	m_nCollapseAnimTime = -1;
	m_nCollapseDelay = 0;
	float fPosFactor = GetRandFloat_MinMax(1.0f, 3.0f);
	m_fLayerFactor = 0.9f/fPosFactor;

	fPosFactor = ((1.0f/fPosFactor) - 0.333f)*1.0f/0.668f;
	ASSERT(fPosFactor >= 0.0f && fPosFactor <= 1.0f);

	m_nLightPattern = rand()%15;

	m_fOriginalX = nOriginX;

	InitXYWH(
		  ConvertOriginalToNormalizedPos_Ex(m_fOriginalX)
		, CITY_YMIN + fPosFactor*(CITY_YMAX - CITY_YMIN)
		, GetRandFloat_MinMax(CITY_WMIN, CITY_WMAX)
		, GetRandFloat_MinMax(CITY_HMIN, CITY_HMAX)
		);
	
	float fLife = 1.0f;
	float fStrength = 1.0f;
	InitHelper(fLife, fStrength, OBJTYPE_PLAYERMISSILE, OWNER_PLAYER1);
	InitLivingObjectFlags(0x00);	
}

/*-------------------------------------------------------------------------*/
/*------------------------------ Is Collapsing ----------------------------*/
/*-------------------------------------------------------------------------*/
bool CCityRect::IsCollapsing()
{
	return (m_nCollapseAnimTime >= 0);
}

/*-------------------------------------------------------------------------*/
/*---------------------------------- Move ---------------------------------*/
/*-------------------------------------------------------------------------*/
/*virtual*/ int CCityRect::Move()
{
	if (m_nCollapseDelay > 0)
	{
		m_nCollapseDelay--;
	}
	else if (m_nCollapseAnimTime > 0)
	{
		m_nCollapseAnimTime--;
		_pQuake->SetQuakeIfLower(QUAKE_SMALLTIME);

		m_fDy = 0.1f + 0.6f*m_fLayerFactor;
		float fRandDx = 1.0f;
		m_fDx = GetRandFloat_MinMax(-fRandDx, fRandDx);
		OriginalMove();
	}
	UpdateScreenPos();
	return 0;
}

/*-------------------------------------------------------------------------*/
/*-------------------------- Check Asteroid Impact ------------------------*/
/*-------------------------------------------------------------------------*/
void CCityRect::DoAsteroidImpact(CMyLivingObject* pAsteroid)
{
	float fWaveShockW = 96.0f;
	
	if (!IsCollapsing()
		&& IsColliding_Pos(pAsteroid->m_fX, pAsteroid->m_fY, fWaveShockW, 2.5f*fWaveShockW))
	{
		m_nCollapseDelay = rand()%16;
		m_nCollapseAnimTime = 120 + rand()%(120 + (int)m_fH/5);
	}
}

/*-------------------------------------------------------------------------*/
/*---------------------------------- Draw ---------------------------------*/
/*-------------------------------------------------------------------------*/
/*virtual*/ int CCityRect::Draw()	
{
	float fAlpha = 0.2f + 0.35f*m_fLayerFactor;
	
	if (m_nCollapseAnimTime >= 0)
	{
		fAlpha *= CalcLinear_Int(m_nCollapseAnimTime, 0, 64);
	}

	float fDiffY = 16.0f*(0.5f + 0.5f*get_dy_a360_hq(0.75f*m_fX + m_fY + 360.0f*_nTotalExeTime/BEAT_PERIOD));

	DrawMonoQuad4f(m_fX, m_fY-0.5f*fDiffY, m_fW, m_fH+fDiffY
		,  0.0f, 0.0f, 0.0f, fAlpha);

	return 0;
}

